# -*- coding: utf-8 -*-

from cpskin.locales import CPSkinMessageFactory as _
from plone.dexterity.content import Container
from plone.supermodel import model
from zope import schema
from zope.interface import implementer


class IProcedure(model.Schema):
    """ Marker interface and Dexterity Python Schema for Procedure
    """

    e_guichet = schema.TextLine(title=_(u"E-Guichet"), required=False)


@implementer(IProcedure)
class Procedure(Container):
    """
    """



# -*- coding: utf-8 -*-

from plone.app.textfield import RichText
from plone.dexterity.content import Container
from plone.supermodel import model
from zope import schema
from zope.interface import implementer


from cpskin.locales import CPSkinMessageFactory as _


class IProduct(model.Schema):
    """ Marker interface and Dexterity Python Schema for Product
    """

    price = schema.TextLine(
        title=_(u'Price'),
        required=False
    )

    color = schema.Text(
        title=_(u'Color(s)'),
        required=False
    )

    size = schema.Text(
        title=_(u'Size(s)'),
        required=False
    )

    informations = RichText(
        title=_(u'Informations'),
        required=False
    )


@implementer(IProduct)
class Product(Container):
    """
    """



# -*- coding: utf-8 -*-

from Products.Five import BrowserView
from plone.app.contenttypes.behaviors.leadimage import ILeadImage


class ProductView(BrowserView):
    """
    """

    def available(self):
        scale_context = ILeadImage(self.context)
        return True if scale_context.image else False



# -*- coding: utf-8 -*-
from plone.app.robotframework.testing import REMOTE_LIBRARY_BUNDLE_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.testing import z2

import cpskin.contenttypes


class CpskinContenttypesLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load any other ZCML that is required for your tests.
        # The z3c.autoinclude feature is disabled in the Plone fixture base
        # layer.
        import plone.app.dexterity
        self.loadZCML(package=plone.app.dexterity)
        self.loadZCML(package=cpskin.contenttypes)

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'cpskin.contenttypes:default')


CPSKIN_CONTENTTYPES_FIXTURE = CpskinContenttypesLayer()


CPSKIN_CONTENTTYPES_INTEGRATION_TESTING = IntegrationTesting(
    bases=(CPSKIN_CONTENTTYPES_FIXTURE,),
    name='CpskinContenttypesLayer:IntegrationTesting',
)


CPSKIN_CONTENTTYPES_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(CPSKIN_CONTENTTYPES_FIXTURE,),
    name='CpskinContenttypesLayer:FunctionalTesting',
)


CPSKIN_CONTENTTYPES_ACCEPTANCE_TESTING = FunctionalTesting(
    bases=(
        CPSKIN_CONTENTTYPES_FIXTURE,
        REMOTE_LIBRARY_BUNDLE_FIXTURE,
        z2.ZSERVER_FIXTURE,
    ),
    name='CpskinContenttypesLayer:AcceptanceTesting',
)



# -*- coding: utf-8 -*-

from imio.behavior.teleservices.behaviors.ts_procedure import ITsProcedure
from plone.app.contenttypes.behaviors.leadimage import ILeadImage
from plone.dexterity.browser import add
from plone.dexterity.browser import edit
from plone.dexterity.interfaces import IDexterityFTI
from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import queryUtility


class ProcedureView(BrowserView):
    """
    """
    index = ViewPageTemplateFile('templates/procedure.pt')

    def has_procedures(self):
        # Get this attr thanks to behavior : imio.behavior.teleservices.
        if getattr(self.context, "procedures", None) is None:
            return False
        else:
            return True

    def print_basic_e_guichet_field(self):
        if ITsProcedure.providedBy(self.context) is True:
            # Don't print basic e_guichet field
            return False
        elif self.context.e_guichet is None:
            # Don't print basic e_guichet field
            return False
        else:
            return True

    def available(self):
        scale_context = ILeadImage(self.context)
        return True if scale_context.image else False


class ProcedureEditForm(edit.DefaultEditForm):
    # if behavior is install so we hide default e_guichet field.
    def updateWidgets(self):
        super(ProcedureEditForm, self).updateWidgets()
        if ITsProcedure.providedBy(self.context):
            self.widgets['e_guichet'].mode = 'hidden'


class CustomAddForm(add.DefaultAddForm):
    portal_type = 'Procedure'

    def updateWidgets(self):
        super(CustomAddForm, self).updateWidgets()
        # When we add new contentype object, our behavior doesn't exist yet
        # So we search ITsProcedure in FTI
        if self.has_behavior("Procedure", "imio.behavior.teleservices.behaviors.ts_procedure.ITsProcedure"):
            self.widgets['e_guichet'].mode = 'hidden'

    def has_behavior(self, type_name, behavior_name):
        """Check if a behavior is on portal_type"""
        fti = queryUtility(IDexterityFTI, name=type_name)
        if not fti:
            return
        behaviors = list(fti.behaviors)
        if behavior_name not in behaviors:
            return False
        else:
            return True


class AddView(add.DefaultAddView):
    form = CustomAddForm



# -*- coding: utf-8 -*-
from plone import api

import logging


logger = logging.getLogger("cpskin.core")


def apply_new_default_view(context):
    catalog = api.portal.get_tool("portal_catalog")
    brains = catalog(
        {"object_provides": "cpskin.contenttypes.content.procedure.IProcedure"}
    )
    for brain in brains:
        obj = brain.getObject()
        obj.setLayout("procedure_view")
        obj.reindexObject()

def no_more_procedure_view(context):
    catalog = api.portal.get_tool("portal_catalog")
    brains = catalog(
        {"object_provides": "cpskin.contenttypes.content.procedure.IProcedure"}
    )
    for brain in brains:
        obj = brain.getObject()
        obj.setLayout("view")
        obj.reindexObject()



# -*- coding: utf-8 -*-
from cpskin.contenttypes.content.product import IProduct  # NOQA E501
from cpskin.contenttypes.testing import CPSKIN_CONTENTTYPES_INTEGRATION_TESTING  # noqa
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.dexterity.interfaces import IDexterityFTI
from zope.component import createObject
from zope.component import queryUtility

import unittest




class ProductIntegrationTest(unittest.TestCase):

    layer = CPSKIN_CONTENTTYPES_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def test_ct_product_schema(self):
        fti = queryUtility(IDexterityFTI, name='Product')
        schema = fti.lookupSchema()
        self.assertEqual(IProduct, schema)

    def test_ct_product_fti(self):
        fti = queryUtility(IDexterityFTI, name='Product')
        self.assertTrue(fti)

    def test_ct_product_factory(self):
        fti = queryUtility(IDexterityFTI, name='Product')
        factory = fti.factory
        obj = createObject(factory)

        self.assertTrue(
            IProduct.providedBy(obj),
            u'IProduct not provided by {0}!'.format(
                obj,
            ),
        )

    def test_ct_product_adding(self):
        setRoles(self.portal, TEST_USER_ID, ['Contributor'])
        obj = api.content.create(
            container=self.portal,
            type='Product',
            id='product',
        )

        self.assertTrue(
            IProduct.providedBy(obj),
            u'IProduct not provided by {0}!'.format(
                obj.id,
            ),
        )

    def test_ct_product_globally_addable(self):
        setRoles(self.portal, TEST_USER_ID, ['Contributor'])
        fti = queryUtility(IDexterityFTI, name='Product')
        self.assertTrue(
            fti.global_allow,
            u'{0} is not globally addable!'.format(fti.id)
        )

    def test_ct_product_filter_content_type_false(self):
        setRoles(self.portal, TEST_USER_ID, ['Contributor'])
        fti = queryUtility(IDexterityFTI, name='Product')
        portal_types = self.portal.portal_types
        parent_id = portal_types.constructContent(
            fti.id,
            self.portal,
            'product_id',
            title='Product container',
         )
        self.parent = self.portal[parent_id]
        obj = api.content.create(
            container=self.parent,
            type='Document',
            title='My Content',
        )
        self.assertTrue(
            obj,
            u'Cannot add {0} to {1} container!'.format(obj.id, fti.id)
        )



# -*- coding: utf-8 -*-

from plone import api
from plone.app.layout.navigation.interfaces import INavigationRoot
from plone.app.textfield.interfaces import IRichText
from plone.app.textfield.value import RichTextValue
from plone.namedfile.file import NamedBlobImage
from Products.CMFPlone.utils import safe_unicode
from z3c.relationfield.relation import RelationValue
from zope import component
from zope.intid.interfaces import IIntIds
from zope.schema import getFieldsInOrder

import copy
import logging
import Missing
import unidecode


logger = logging.getLogger("cpskin.contenttypes")


def post_install(context):
    """We need to migrate existing TTW content types if any"""
    migrate_product(context)
    migrate_procedures(context)


def get_brains():
    # Récupérer les mots clés "je trouve"
    # types to migrate
    portal_types = api.portal.get_tool(name="portal_types")
    existing_brains = []
    if "demarche" in portal_types:
        existing_brains = api.content.find(portal_type="demarche")
    else:
        existing_document_brains = api.content.find(portal_type="Document")
        for brain in existing_document_brains:
            if brain.isearchTags == Missing.Value or brain.isearchTags is None:
                continue
            for tag in brain.isearchTags:
                if "DEMARCHE" in unidecode.unidecode(safe_unicode(tag)).upper():
                    existing_brains.append(brain)
    return existing_brains


def migrate_procedures(context):
    # keywords (DEMARCHE)
    # history_for_related_items = []
    portal_types = api.portal.get_tool(name="portal_types")
    existing_brains = []
    # new type Procedure.
    if "Procedure" not in portal_types:
        context.runImportStepFromProfile(
            "profile-cpskin.contenttypes:default", "typeinfo"
        )

    existing_brains = get_brains()

    for brain in existing_brains:
        old_procedure = brain.getObject()
        container = old_procedure.aq_parent
        # No procedure in PloneSite root
        # In each site, there is a document with a keyword
        # "démarche administrative" that stores the keyword.
        if INavigationRoot.providedBy(container):
            continue
        new_id = old_procedure.id
        temp_id = "new-{0}".format(new_id)
        # new type "Procedure"
        if portal_types.get("Procedure") not in container.allowedContentTypes():
            container.locally_allowed_types.append("Procedure")
            # container.allowedContentTypes().append(portal_types.get("Procedure"))
        new_procedure = api.content.create(
            container=container,
            type="Procedure",
            id=temp_id,
            title=old_procedure.Title(),
            safe_id=False,
        )
        new_procedure.description = old_procedure.Description()
        if hasattr(old_procedure, "e_guichet"):
            new_procedure.e_guichet = old_procedure.e_guichet
        if hasattr(old_procedure, "text"):
            if old_procedure.text is not None:
                new_procedure.text = RichTextValue(
                    raw=old_procedure.text.raw,
                    mimeType=old_procedure.text.mimeType,
                    outputMimeType=old_procedure.text.outputMimeType,
                )
        elif hasattr(old_procedure, "texte"):
            if old_procedure.texte is not None:
                new_procedure.text = RichTextValue(
                    raw=old_procedure.texte.raw,
                    mimeType=old_procedure.texte.mimeType,
                    outputMimeType=old_procedure.texte.outputMimeType,
                )
        elif hasattr(old_procedure, "contenu"):
            if old_procedure.contenu is not None:
                new_procedure.text = RichTextValue(
                    raw=old_procedure.contenu.raw,
                    mimeType=old_procedure.contenu.mimeType,
                    outputMimeType=old_procedure.contenu.outputMimeType,
                )
        else:
            # find RichText on "demarche" (old_procedure)
            # get first and set into new_procedure.text
            attr = None
            if "demarche" in portal_types:
                dem = portal_types["demarche"]
                fields = getFieldsInOrder(dem.lookupSchema())
                for name, field in fields:
                    if IRichText.providedBy(field):
                        attr = name
                        break
            if attr is not None:
                if old_procedure.__dict__.get(attr) is not None:
                    new_procedure.text = RichTextValue(
                        raw=old_procedure.__dict__.get(attr).raw,
                        mimeType=old_procedure.__dict__.get(attr).mimeType,
                        outputMimeType=old_procedure.__dict__.get(attr).outputMimeType,
                    )
        new_procedure.effective_date = old_procedure.effective_date
        new_procedure.creation_date = old_procedure.creation_date
        modification_date = old_procedure.modification_date
        new_procedure.workflow_history = old_procedure.workflow_history
        new_procedure.creators = old_procedure.creators
        new_procedure.language = old_procedure.language
        if hasattr(old_procedure, "hiddenTags"):
            new_procedure.hiddenTags = copy.deepcopy(old_procedure.hiddenTags)
        if hasattr(old_procedure, "iamTags"):
            new_procedure.iamTags = copy.deepcopy(old_procedure.iamTags)
        if hasattr(old_procedure, "isearchTags"):
            new_procedure.isearchTags = copy.deepcopy(old_procedure.isearchTags)
        if hasattr(old_procedure, "standardTags"):
            new_procedure.standardTags = copy.deepcopy(old_procedure.standardTags)

        # restore workflow state
        state = api.content.get_state(old_procedure)
        api.content.transition(obj=new_procedure, to_state=state)

        new_procedure.reindexObject()

        wf = new_procedure.portal_workflow.getWorkflowsFor(new_procedure)[0]
        wf.updateRoleMappingsFor(new_procedure)
        new_procedure.reindexObject()

        new_procedure.modification_date = modification_date
        new_procedure.reindexObject(idxs=["modified"])
        logger.info("Migrated procedure {0}".format(new_procedure.absolute_url()))

    get_related_items()
    # Remove old procedures after conversion process
    # because of old procedures relatedItems !
    remove_old_procedures()
    if "demarche" in portal_types:
        del portal_types["demarche"]


def get_related_items():
    existing_brains = get_brains()
    for brain in existing_brains:
        old_procedure = brain.getObject()
        container = old_procedure.aq_parent
        if INavigationRoot.providedBy(container):
            continue
        new_procedure = container.restrictedTraverse(
            "{}/{}".format(
                "/".join(container.getPhysicalPath()),
                "new-{}".format(old_procedure.getId()),
            )
        )
        if hasattr(old_procedure, "relatedItems") and old_procedure.relatedItems != []:
            for rel in old_procedure.relatedItems:
                if (
                    rel.to_object is not None
                    and rel.to_object.getTypeInfo().getId() == "demarche"
                ):
                    intids = component.getUtility(IIntIds)
                    relobj = rel.to_object
                    re = relobj.aq_parent.restrictedTraverse(
                        "{}/{}".format(
                            "/".join(relobj.aq_parent.getPhysicalPath()),
                            "new-{}".format(relobj.getId()),
                        )
                    )
                    old_procedure.relatedItems.append(RelationValue(intids.getId(re)))
                    old_procedure.relatedItems.remove(rel)
                    new_procedure.relatedItems = copy.deepcopy(
                        old_procedure.relatedItems
                    )
                    logger.info(
                        "{0} lien vers {1}".format(
                            new_procedure.absolute_url(), re.absolute_url()
                        )
                    )
        new_procedure.reindexObject()


def remove_old_procedures():
    existing_brains = get_brains()
    for brain in existing_brains:
        old_procedure = brain.getObject()
        container = old_procedure.aq_parent
        if INavigationRoot.providedBy(container):
            continue
        new_procedure = container.restrictedTraverse(
            "{}/{}".format(
                "/".join(container.getPhysicalPath()),
                "new-{}".format(old_procedure.getId()),
            )
        )
        new_id = new_procedure.getId().replace("new-", "")
        api.content.delete(obj=old_procedure, check_linkintegrity=True)
        api.content.rename(obj=new_procedure, new_id=new_id, safe_id=True)


def migrate_product(context):
    """Existing TTW 'produit' content type migration"""
    portal_types = api.portal.get_tool(name="portal_types")
    if "produit" not in portal_types:
        return

    existing_brains = api.content.find(portal_type="produit")
    for brain in existing_brains:
        old_product = brain.getObject()
        container = old_product.aq_parent
        new_id = old_product.id
        temp_id = "new-{0}".format(new_id)
        new_product = api.content.create(
            container=container,
            type="Product",
            id=temp_id,
            title=old_product.Title(),
            safe_id=False,
        )

        # copy standard fields / values
        new_product.description = old_product.Description()
        new_product.price = old_product.prix
        new_product.size = old_product.taille_s
        new_product.color = old_product.couleur_s
        if old_product.informations:
            new_product.informations = RichTextValue(
                raw=old_product.informations.raw,
                mimeType=old_product.informations.mimeType,
                outputMimeType=old_product.informations.outputMimeType,
            )
        new_product.effective_date = old_product.effective_date
        new_product.creation_date = old_product.creation_date
        modification_date = old_product.modification_date
        new_product.workflow_history = old_product.workflow_history
        new_product.creators = old_product.creators
        new_product.language = old_product.language

        # copy leadimage
        old_image = old_product.image
        if old_image:
            filename = old_image.filename
            old_image_data = old_image.data
            namedblobimage = NamedBlobImage(data=old_image_data, filename=filename)
            setattr(new_product, "image", namedblobimage)

        # restore workflow state
        state = api.content.get_state(old_product)
        api.content.transition(obj=new_product, to_state=state)

        api.content.delete(obj=old_product, check_linkintegrity=True)
        api.content.rename(obj=new_product, new_id=new_id, safe_id=True)
        new_product.reindexObject()

        wf = new_product.portal_workflow.getWorkflowsFor(new_product)[0]
        wf.updateRoleMappingsFor(new_product)
        new_product.reindexObject()

        new_product.modification_date = modification_date
        new_product.reindexObject(idxs=["modified"])

        logger.info("Migrated product {0}".format(new_product.absolute_url()))

    del portal_types["produit"]



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from cpskin.contenttypes.testing import CPSKIN_CONTENTTYPES_INTEGRATION_TESTING  # noqa

import unittest


class TestSetup(unittest.TestCase):
    """Test that cpskin.contenttypes is properly installed."""

    layer = CPSKIN_CONTENTTYPES_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer['portal']
        self.installer = api.portal.get_tool('portal_quickinstaller')

    def test_product_installed(self):
        """Test if cpskin.contenttypes is installed."""
        self.assertTrue(self.installer.isProductInstalled(
            'cpskin.contenttypes'))

    def test_browserlayer(self):
        """Test that ICpskinContenttypesLayer is registered."""
        from cpskin.contenttypes.interfaces import (
            ICpskinContenttypesLayer)
        from plone.browserlayer import utils
        self.assertIn(
            ICpskinContenttypesLayer,
            utils.registered_layers())
